/*
 *  Copyright 2008-2009 Olivier PARISOT
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.doopyon.musaterm.client;

import org.doopyon.musaterm.client.term.*;
import com.google.gwt.user.client.ui.*;


/**
 * A GWT implementation of a terminal.
 * 
 * @author Olivier PARISOT
 */
public final class GWTTerminal extends Terminal 
{
	//
	// Static fields
	//

	/** Splash message. */
	private static final String SPLASH_MESSAGE="Welcome to the musaterm.\n\n";
	  
	/** Prompt message. */
	private static final String PROMPT_MESSAGE="you@musaterm> ";

	/** End message. */
	private static final String END_MESSAGE="This terminal is closed!";	
	
	  
	//
	// Instance fields
	//
	
	/** */
	private final Panel currentPanel;
	
	/** */
	private TextArea currentTextArea;    
	  
	
	//
	// Constructor
	//
	
	/**
	 * Constructor.
	 */
	GWTTerminal(AbstractCommandLineExecutor commandManager, Panel currentPanel)
	{
		super(commandManager);
		this.currentPanel=currentPanel;		
	}
	
	
	//
	// Instance methods
	//

	/**
	 * {@inheritDoc} 
	 */
	public void init()
	{
		print(SPLASH_MESSAGE);
		printNewPrompt();
	}	
	
	/**
	 * {@inheritDoc} 
	 */
	public void clear()
	{
		currentPanel.clear();
		print(SPLASH_MESSAGE);
		// @FIXME: avoid "focus lost" bug!!
	}

	/**
	 * {@inheritDoc} 
	 */
	public void close()
	{
		super.close();
		print(END_MESSAGE);
	}	
	
	/**
	 * {@inheritDoc} 
	 */	
	public void print(String text)
	{
	    final HorizontalPanel panel=buildHorizontalPanel();
	    panel.add(new Label(text));    
	    currentPanel.add(panel);  		
	}
	
	/**
	 * 
	 */
	private void printNewPrompt()
	{
	    if (currentTextArea!=null) currentTextArea.setEnabled(false);    
	    
	    currentTextArea=new TextArea();
	    currentTextArea.setText("");
	    currentTextArea.setCharacterWidth(80);
	    currentTextArea.setVisibleLines(1);
	    currentTextArea.addKeyboardListener(new WebKeyboardListener());
	  
	    final HorizontalPanel panel=buildHorizontalPanel();
	    panel.add(new Label(PROMPT_MESSAGE));    
	    panel.add(currentTextArea);
	    currentPanel.add(panel);
	    
	    currentTextArea.setFocus(true);		
	}
	
	/**
	 * Build a panel.
	 * @return a panel
	 */
	private HorizontalPanel buildHorizontalPanel()
	{
	    final HorizontalPanel panel=new HorizontalPanel();
	    panel.setWidth("100%");
	    panel.setHorizontalAlignment(VerticalPanel.ALIGN_LEFT);
	    return panel;
	}
	
	
	//
	// Internal classes
	//
	
	/**
	 * Keyboard listener.
	 */
	private final class WebKeyboardListener implements KeyboardListener
	{
		//
		// Instance methods
		//
		
		/**
		 * {@inheritDoc} 
		 */	
		public void onKeyDown(Widget sender, char keyCode, int modifiers) {}

		/**
		 * {@inheritDoc} 
		 */	
		public void onKeyPress(Widget sender, char keyCode, int modifiers) 
		{
	        if (keyCode==(char)KEY_ENTER)
	        {
	        	final TextArea ta=(TextArea)sender;
	        	ta.cancelKey(); 
	        	executeCommandLine(ta.getText());
	        	if (!isClosed()) printNewPrompt();
	        }			
		}

		/**
		 * {@inheritDoc} 
		 */	
		public void onKeyUp(Widget sender, char keyCode, int modifiers) {}		
	}

}
