/*
 *  Copyright 2008-2009 Olivier PARISOT
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.doopyon.musaterm.client;

import org.doopyon.musaterm.client.term.*;
import org.doopyon.musaterm.client.term.common.*;
import com.google.gwt.user.client.ui.*;


/**
 *
 * 
 * @author Olivier PARISOT
 */
public final class GWTTerminalController 
{
	//
	// Static fields
	//
	
	/** */
	private final static int MAX_TAB_COUNT=5;
	
	
	//
	// Instance fields
	//
	
	/** */
	private final Panel mainPanel;
	
	/** */
	private int tabCount;
	/** */
	private final DecoratedTabPanel tabPanel;
	/** */
	private final TabListener listener;
	
	
	//
	// Constructor
	//
	
	/**
	 * Constructor.
	 */
	public GWTTerminalController(Panel mainPanel)
	{
		this.mainPanel=mainPanel;
		this.tabCount=1;
		this.tabPanel=new DecoratedTabPanel();
		this.listener=new MoreTabListener(this);		
	}
	
	
	//
	// Instance methods
	//
	
	/**
	 * 
	 */
	private void addTerminal()
	{
		if (tabCount<=MAX_TAB_COUNT)
		{
			final VerticalPanel firstTerminalPanel=new VerticalPanel();
			tabPanel.add(firstTerminalPanel,"musaterm #"+(tabCount++));	    
			final AbstractTerminal terminal=new GWTTerminal(new CommandLineExecutor(new BasicCommandsProvider()),firstTerminalPanel);
			terminal.init();
		}
		else
		{
			showWarningDialogBox("You can't open more than "+MAX_TAB_COUNT+" terminals!");
		}
		tabPanel.selectTab((tabCount-1));		
	}
	
	/**
	 * 
	 */
	public void init()
	{
	  	/* 'decorate' the tab panel */
	    tabPanel.setWidth("50%");
	    tabPanel.setAnimationEnabled(true);
	    tabPanel.addTabListener(listener);	    
	    	    
	    /* add a 'fake' tab used to add others panels */
	    tabPanel.add(new VerticalPanel(),"+");
	    	    
	    mainPanel.add(tabPanel);
	    
	    /* add a first terminal panel */
	    addTerminal();
	}
	
	/**
	 * 
	 */
	private void showWarningDialogBox(String warningMessage)
	{
		final DialogBox db=new DialogBox(false);
		db.setText("Warning");
		
		final VerticalPanel dbContents=new VerticalPanel();
	    final HTML message=new HTML(warningMessage);
	    dbContents.add(message);
	    
	    final SimplePanel holder=new SimplePanel();
	    final Button closeButton=new Button("Close",new ClickListener() 
		{
				public void onClick(Widget sender) {db.hide();}
		});
	    
	    holder.add(closeButton);
	    dbContents.add(holder);
	    
	    db.setWidget(dbContents);
			
		db.setAnimationEnabled(true);
        db.center();			
		db.show();		
	}
	
	
	//
	// Private classes
	//
	
	/**
	 *
	 * 
	 * @author Olivier PARISOT	 
	 */
	private final class MoreTabListener implements TabListener
	{
		//
		// Instance fields
		//
		
		/** */
		private final GWTTerminalController ctrler;
		
		
		//
		// Constructor
		//
		
		/**
		 * Constructor.
		 */
		MoreTabListener(GWTTerminalController ctrler)
		{
			this.ctrler=ctrler;
		}
		
		
		//
		// Instance methods
		//
		
		/**
		 * {@inheritDoc}
		 */
		public boolean onBeforeTabSelected(SourcesTabEvents sender,int tabIndex) 
		{
			return true;
		}

		/**
		 * {@inheritDoc}
		 */
		public void onTabSelected(SourcesTabEvents sender,int tabIndex) 
		{
			if (tabIndex==0) ctrler.addTerminal();			
		}
		
	}	
}
