/*
 *  Copyright 2008-2009 Olivier PARISOT
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.doopyon.musaterm.client.term;

import java.util.*;


/**
 * Command line parser.
 *
 * @author Olivier PARISOT
 */
public final class CommandLineParser implements AbstractCommandLineParser 
{
	//
	// Instance fields
	//
	
	/** The available commands. */
	private final Map<String,Command> map=new HashMap<String,Command>();
	
	
	//
	// Constructor
	//
	
	/**
	 * Constructor.
	 */
	public CommandLineParser(AbstractCommandsProvider provider)
	{
		for (Command c:provider.getCommands()) map.put(c.getName(),c);
	}
	
	
	//
	// Instance methods
	//	
	
	/**
	 * {@inheritDoc}
	 */
	public Command parseCommand(String commandLine)
	{		
		final int indexOfSeparator=commandLine.indexOf(" ");
		return (indexOfSeparator<0)?map.get(commandLine)
								   :map.get(commandLine.substring(0,indexOfSeparator));
	}
	
	/**
	 * {@inheritDoc}
	 */
	public ArgumentsList parseArgument(String commandLine)
	{
		Collection<String> c=null;
		final int indexOfSeparator=commandLine.indexOf(" ");
		if (indexOfSeparator>0)
		{
			c=new ArrayList<String>(1);
			c.add(commandLine.substring(indexOfSeparator));
		}
		return new ArgumentsList(c);
	}	
}
